%% An implementation of change point detection using Adams and McKay
%Copyright Ananth Ranganathan (Honda Research Institute) 2010

function [changepts, particle_history] = changepointdetect(data,params,...
    logpdf,update_stats,truncate)
     
prior = params.prior;                           %prior parameters
lh_params = params.likelihood_params;   %data likelihood parameters
lambda_gap = params.lambda_gap;

%particle id
p_id = 1;

%list for current changepoint options t=0 is a changepoint
particles = struct;
particles(1).id = p_id;
particles(1).r = 0;
particles(1).stats = prior;
particles(1).data = [];
particles(1).joint = 0;

r_posterior(1).id = particles(1).id;
r_posterior(1).r = particles(1).r;
r_posterior(1).p = particles(1).joint;
r_posterior(1).stats = particles(1).stats;

particle_history = struct;
particle_history(1).list = r_posterior;

for i = 1:size(data,1),
    if mod(i,10)==0,
        fprintf(1,'Processing datapoint %d\n',i);
    end
    r_posterior = struct;
    x = data(i,:);
    particles_new = struct;
    %the growth cases
    for j = 1:size(particles,2),
        p = particles(j);
        particles_new(j).id = p.id;
        pi = logpdf(x, p.stats, lh_params);
        
        growth_prob = p.joint + log(1-1/lambda_gap) + pi;
        particles_new(j).r = p.r+1;
        particles_new(j).joint = growth_prob;
        particles_new(j).stats = update_stats(x,p.stats,p.data,params);
        particles_new(j).data = [p.data;x];
        
    end
    
    %the change point case
    cp_prob = [];
    for k = 1:size(particles,2),
        p2 = particles(k);
        pi = logpdf(x, p2.stats, lh_params);
        cp_prob = [cp_prob  p2.joint+log(1/lambda_gap)+pi];
    end
    m = max(cp_prob);
    cp_prob = m + log(sum(exp(cp_prob-m)));
    
    p_id = p_id+1;
    particles_new(j+1).id = p_id;
    particles_new(j+1).r = 0;
    particles_new(j+1).joint = cp_prob;
    particles_new(j+1).stats = prior;
    particles_new(j+1).data = [];   
    
    list = [particles_new.joint];
    m = max(list);
    evidence = m + log(sum(exp(list-m)));
    
    for j = 1:size(particles_new,2),
        r_posterior(j).id = particles_new(j).id;
        r_posterior(j).r = particles_new(j).r;
        r_posterior(j).p = particles_new(j).joint - evidence;
        r_posterior(j).stats = particles_new(j).stats;
    end
    
    if truncate>0,
        indices = exp([r_posterior.p]) > truncate;
        particle_history(i+1).list = r_posterior(indices);
        particles = particles_new(indices);
    else
        particle_history(i+1).list = r_posterior;
        particles = particles_new;
    end
end

changepts = r_posterior;