function [ ] = CalculateDictionarySparse( imageFileList, dataBaseDir, featureSuffix, dictionarySize, numTextonImages, canSkip )
%
% Create the sparse texton dictionary according to Yang et al, CVPR 2009
%
% Parameters are same as for CalculateDictionary.m

fprintf('Building Dictionary\n\n');

%% parameters

reduce_flag = 1;
ndata_max = 75000;

if(nargin<4)
    dictionarySize = 200
end

if(nargin<5)
    numTextonImages = 50
end

if(nargin<6)
    canSkip = 0
end

if(numTextonImages > size(imageFileList,1))
    numTextonImages = size(imageFileList,1);
end

outFName = fullfile(dataBaseDir, sprintf('dictionarySparse_%d.mat', dictionarySize));

if(size(dir(outFName),1)~=0 && canSkip)
    fprintf('Dictionary file %s already exists.\n', outFName);
    return;
end
    

%% load file list and determine indices of training images

inFName = fullfile(dataBaseDir, 'f_order.txt');
if ~isempty(dir(inFName))
    R = load(inFName, '-ascii');
    if(size(R,1)~=size(imageFileList,1))
        R = randperm(size(imageFileList,1));
        sp_make_dir(inFName);
        save(inFName, 'R', '-ascii');
    end
else
    R = randperm(size(imageFileList,1));
    sp_make_dir(inFName);
    save(inFName, 'R', '-ascii');
end

training_indices = R(1:numTextonImages);

%% load all SIFT descriptors

sift_all = [];

for f = 1:numTextonImages    
    
    imageFName = imageFileList{training_indices(f)};
    [dirN base] = fileparts(imageFName);
    baseFName = fullfile(dirN, base);
    inFName = fullfile(dataBaseDir, sprintf('%s%s', baseFName, featureSuffix));

    load(inFName, 'features');
    ndata = size(features.data,1);

    sift_all = [sift_all; features.data];
    fprintf('Loaded %s, %d descriptors, %d so far\n', inFName, ndata, size(sift_all,1));
end

fprintf('\nTotal descriptors loaded: %d\n', size(sift_all,1));

ndata = size(sift_all,1);    
if (reduce_flag > 0) & (ndata > ndata_max)
    fprintf('Reducing to %d descriptors\n', ndata_max);
    p = randperm(ndata);
    sift_all = sift_all(p(1:ndata_max),:);
end
        
%% perform clustering
options = foptions;
options(1) = 1; % display
options(2) = 1;
options(3) = 0.1; % precision
options(5) = 1; % initialization
options(14) = 100; % maximum iterations

%% run sparse learning algorithm

fprintf('\nRunning sparse learning\n');
%[U, V] = clusterSparse(sift_all, dictionarySize);
beta = 1e-2;
epsilon = 0;
num_iters = 2;
batch_size = 1000;
pars.display_images = true;
pars.save_every = 200;

[B S stat] = sparse_coding(sift_all', dictionarySize, beta, 'L1',...
                           epsilon, num_iters,batch_size, 'blah_fname', pars);
U = S'; dictionary = B';
                       

fprintf('Saving texton dictionary\n');
sp_make_dir(outFName);
save(outFName, 'dictionary');

end