%Sparse clustering using FPC_AS
%U contains membership
%V contains centers
function [retU, retV] = clusterSparse(indata, ncenters, niters)

if nargin < 2 || nargin > 3,
    error('clusterSparse must have at least two arguments and no more than three');
elseif nargin == 2,
    niters = 10;
end

data = indata;
[ndata, ndims] = size(data);
V = rand(ncenters, ndims) - 0.5; %rand(ncenters, ndims);
V = V - repmat(mean(V,2), 1, size(V,2));
for i = 1:ncenters,
    V(i,:) = V(i,:)/norm(V(i,:));
end

gamma = 1e-8;
opts.scale_A = 1;
%opts.minK = ncenters;
opts.record = -1;

for n = 1:niters,
    fprintf(1,'Optimization iter %d\n',n);
    
    for m = 1:ndata,
        if mod(m,1000)==0,
            fprintf(1,'Computed U for %d data\n',m);
        end
        [U(m,:), Out] = FPC_AS(ncenters,V',data(m,:)',gamma,[],opts);
    end
    
    V = l2ls_learn_basis_dual(data', U', 1);
    V = V';
end

retV = V;
retU = U;