%Sparse clustering using feature sign algorithm following Lee06nips
%U contains membership
%V contains centers
function [retU, retV] = clusterSparse(indata, ncenters)

global U data;

data = indata;
niters = 10;
[ndata, ndims] = size(data);
V = rand(ncenters, ndims) - 0.5; %rand(ncenters, ndims);
V = V - repmat(mean(V,2), 1, size(V,2));
for i = 1:ncenters,
    V(i,:) = V(i,:)/norm(V(i,:));
end
U = zeros(ndata, ncenters);
gamma = 10.0;

%cvx_quiet(true);

Vold = 0;

noise_var = 1;
sigma = 1;
tol = 0.005;
beta = 0.01;
epsilon = 0.01;

for n = 1:niters,
    fprintf(1,'Optimization iter %d\n',n);
    
    %{
    cvx_clear();
    for i = 1:ndata,
        cvx_begin
            variable u(ncenters);
            minimize ( norm(-u'*V + data(i,:)) + gamma*norm(u,1) );
        cvx_end
        U(i,:) = u;
    end
    U = abs(U);
    %}
    if n==1,
        U = cgf_fitS_sc2(V', data', 'epsL1', noise_var, beta, epsilon,...
            sigma, tol, false, false, false);
        %U = l1ls_featuresign(V', data', gamma);
    else
        %U = cgf_fitS_sc2(V', data', 'epsL1', noise_var, beta, epsilon,...
        %    sigma, tol, false, false, false, U');
        U = l1ls_featuresign(V', data', gamma, U');
    end
    U(find(isnan(U)))=0;
    U = U';
    V = l2ls_learn_basis_dual(data', U', 1);
    V = V';
    %lambda = solveDual()
    %V = inv(U'*U+diag(lambda))*(data'*U)'
    
    %{
    cvx_begin
        variable V(ncenters, ndims);
        minimize ( trace(quad_form(data-U*V, eye(ndata))) )
        subject to
            for k = 1:ncenters,
                norm(V(k,:)) <= 1;
            end
    cvx_end
    %}
    
    %{
    if n>1,
        epsilon = sum(sum((Vold-V).^2))
        if epsilon < 1e-4,
            break;
        end
    end
    Vold = V;
    %}
end

retV = V;
retU = U;

%solve lagrange dual for bases
function lambda = solveDual()
global U;
[ndata, ncenters] = size(U);
fn = @eval;
lambda0 = 10*abs(rand(1,ncenters));
opts = lbfgs_options('iprint', -1, 'maxits', 100, 'factr', 1e3, ...
                     'cb', @lambda_callback);
[lambda,fb,exitflag,userdata] = lbfgs(fn,lambda0,opts);
exitflag

function [f, g] = eval(lambda)
global U data;
[ndata, ncenters] = size(U);

I = eye(ncenters);
L = diag(lambda);
g = zeros(1,ncenters);
c = 1;

tmp1 = inv(U'*U+L);
f = trace(data'*data) - trace(data'*U*tmp1*(data'*U)') - c*sum(lambda); %from Lee06nips (8)
tmp = data'*U*tmp1;
g = (sum((tmp).^2) - c); %from Lee06nips (9)
f = -f;                %lbfgs minimizes while we want to maximize, so negative
g = -g;


function [stop,user_data] = lambda_callback(x,iter,state,user_data,opts)

switch state
    case 'init'
        user_data.f = zeros(1,opts.maxits);
        user_data.x = zeros(length(x),opts.maxits);
        user_data.its = 0;
        fprintf(1,'Initing lbfgs\n');
    case 'iter'
        user_data.f(iter.it) = iter.f;
        user_data.x(:,iter.it) = x;
        user_data.its = user_data.its + 1;

        fprintf('Iteration %4d\tf: %-8.6g\n', iter.it, iter.f);
    case 'done'
        fprintf(1,'Done after %d iterations\n',user_data.its);
        user_data.f = user_data.f(1:user_data.its);
        user_data.x = user_data.x(:,1:user_data.its);
end

stop = 0;