/* C implementation of sp_find_grid.m
 * sift_arr = sp_find_sift_grid(grid_x, grid_y, patch_size, I_orientation, sample_x, sample_y)
 */

#include "mex.h"
#include <string.h>
#include <stdio.h>
#include <matrix.h>
#include <math.h>

/* Main entry function */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    const mxArray *grid_x = prhs[0], *grid_y = prhs[1],
                    *ps = prhs[2], *I_o = prhs[3],
                    *sample_x = prhs[4], *sample_y = prhs[5];
    
    double *sift_arr_p, r, *grid_xp, *grid_yp, cx, cy;
    double x_lo, x_hi, y_lo, y_hi;
    int i, j, k, l, patch_size;
    /* parameters */
    const int num_angles = 8, num_bins = 4, alpha = 9;
    const int num_samples = num_bins * num_bins;
    size_t num_patches;
    double angle_step = 2 * M_PI / num_angles;
    double sample_x_t[num_samples];
    double sample_y_t[num_samples];
    double sample_res, tmp, *sample_xp, *sample_yp, *I_op;
    int sample_px, sample_py, nx, ny, num_pix, el[2], el2[2], el3[3];
    double weights_x, weights_y, dist_px, dist_py, *weightsp=0, sum, *tmpf;
    mxArray *weights, *plhs_inner[1], *prhs_inner[9];
    
    if (nrhs!=6)
    {
        mexErrMsgTxt("sp_find_sift_grid_c has to have 6 input args");
    }
    if(nlhs!=1)
    {
        mexErrMsgTxt("sp_find_sift_grid_c has to have 1 output arg only");
    }
    
    patch_size = mxGetScalar(ps);
    num_patches = mxGetNumberOfElements(grid_x);

    //allocate output
    plhs[0] = mxCreateDoubleMatrix(num_patches,num_samples * num_angles,mxREAL);
    sift_arr_p = mxGetPr(plhs[0]);
    
    //allocate inner loop function inputs
    prhs_inner[0] = I_o;
    for(i=2;i<9;i++)
        prhs_inner[i] = mxCreateDoubleMatrix(1,1,mxREAL);
    tmpf = mxGetPr(prhs_inner[2]); *tmpf = (double)num_angles;
    tmpf = mxGetPr(prhs_inner[3]); *tmpf = (double)num_samples;
    
    grid_xp = mxGetPr(grid_x);
    grid_yp = mxGetPr(grid_y);
    sample_xp = mxGetPr(sample_x);
    sample_yp = mxGetPr(sample_y);
    I_op = mxGetPr(I_o);
    /* for all patches */
    for(i=0;i<num_patches;i++) 
    {
        r = patch_size/2;
        cx = grid_xp[i] + r - 0.5;
        cy = grid_yp[i] + r - 0.5;

        /* find coordinates of sample points (bin centers) */
        for(j=0;j<num_samples;j++)
        {
            sample_x_t[j] = sample_xp[j] * r + cx;
            sample_y_t[j] = sample_yp[j] * r + cy;
        }
        sample_res = sample_y_t[1] - sample_y_t[0];
        
        /* find window of pixels that contributes to this descriptor */
        x_lo = grid_xp[i];
        x_hi = grid_xp[i] + patch_size - 1;
        y_lo = grid_yp[i];
        y_hi = grid_yp[i] + patch_size - 1;
        
        nx = (int)(x_hi-x_lo+1);
        ny = (int)(y_hi-y_lo+1);
        num_pix = nx*ny;
        
        if(!weightsp)
        {
            weights = mxCreateDoubleMatrix(num_pix,num_samples,mxREAL);
            weightsp = mxGetPr(weights);
        }
        
        for(j=0;j<num_pix;j++)
        {
            el[0] = j;
            /* find coordinates of pixels */
            sample_px = j/ny + x_lo;
            sample_py = j%ny + y_lo;
            
            for(k=0;k<num_samples;k++)
            {
                el[1] = k;
                /* find (horiz, vert) distance between each pixel and each grid sample */
                dist_px = fabs(sample_px - sample_x_t[k]);
                dist_py = fabs(sample_py - sample_y_t[k]);
                weights_x = dist_px/sample_res;
                weights_x = (weights_x<=1)?(1-weights_x):0;
                weights_y = dist_py/sample_res;
                weights_y = (weights_y<=1)?(1-weights_y):0;
                weightsp[mxCalcSingleSubscript(weights,2,el)] = weights_x*weights_y;
            }
        }
        /* make sift descriptor */
        prhs_inner[1] = weights;
        tmpf = mxGetPr(prhs_inner[4]); *tmpf = (double)num_pix;
        tmpf = mxGetPr(prhs_inner[5]); *tmpf = x_lo;
        tmpf = mxGetPr(prhs_inner[6]); *tmpf = x_hi;
        tmpf = mxGetPr(prhs_inner[7]); *tmpf = y_lo;
        tmpf = mxGetPr(prhs_inner[8]); *tmpf = y_hi;
        if(mexCallMATLAB(1, plhs_inner, 9, prhs_inner, 
                            "sp_find_sift_grid_inner_loop"))
        {
            mexErrMsgTxt("Call to inner loop function failed!");
        }
        tmpf = mxGetPr(plhs_inner[0]);
        tmp = num_samples * num_angles;
        el[0] = i;
        for(k=0;k<tmp;k++)
        {
            el[1] = k;
            sift_arr_p[mxCalcSingleSubscript(plhs[0],2,el)] = tmpf[k];
        }
        mxDestroyArray(plhs_inner[0]);
        /*
        el2[0] = i;
        for(j=0;j<num_angles;j++)
        {
            el3[2] = j;
            for(l=0;l<num_samples;l++)
            {
                el[1] = l;
                sum = 0;
                for(k=0;k<num_pix;k++)
                {
                    el3[0] = k%ny + y_lo;
                    el3[1] = k/ny + x_lo;
                    tmp = I_op[mxCalcSingleSubscript(I_o,3,el3)];
                    el[0] = k;
                    sum += tmp*weightsp[mxCalcSingleSubscript(weights,2,el)];
                }
                el2[1] = l*num_angles+j;
                sift_arr_p[mxCalcSingleSubscript(plhs[0],2,el2)] = sum;
            }
        }
         */
    }

    mxDestroyArray(weights);
}