%test changepoint detection on a image dataset using Spatial Pyramid
%Histograms as input measurements

%Copyright Ananth Ranganathan (Honda Research Institute) 2010

clear all;

addpath spatial_pyramid_code;
addpath lightspeed;
addpath fastfit;

image_dir = '/Users/ananth/data/place-categorization/video_from_mac/run1run2';
data_dir = '/Users/ananth/data/place-categorization/video_from_mac/run1_data';

diroutput = dir(fullfile(image_dir,'*.png'));
nimages = size(diroutput,1);

imagefiles = cell(nimages,1);
for i = 1:nimages,
    imagefiles{i} = diroutput(i).name;
end

maxImageSize = 240;
dictionarySize = 200;
numTextonImages = nimages;
pyramidLevels = 1;
canSkip = 1;

gridSpacing = 8;
patchSize = 16;
selected = 1:4:nimages;

[base,datan] = fileparts(image_dir);
outf = fullfile(data_dir,sprintf('changepts_%s.mat',datan));
if exist(outf,'file')
    load(outf);
else
    %% build the pyramid
    GenerateSiftDescriptors( imagefiles,image_dir,data_dir,maxImageSize,...
        gridSpacing,patchSize,canSkip);
    filescalc = imagefiles(1:20:end);
    numfilescalc = size(filescalc,1)
    CalculateDictionary(filescalc,data_dir,'_sift.mat',dictionarySize,...
        numfilescalc,canSkip);
    BuildHistograms(imagefiles,data_dir,'_sift.mat',dictionarySize,canSkip);
    [pyramid_all, lengths] = CompilePyramid(imagefiles,data_dir,sprintf('_texton_ind_%d.mat',...
        dictionarySize),dictionarySize,pyramidLevels,canSkip);
    
    data = pyramid_all(selected,:).*repmat(lengths(selected)',1,size(pyramid_all,2));
    ndims = size(data,2);

    %changepoint detection stuff from here
    %prior = polya_fit(data);
    %params.prior = struct('alpha',prior*0.1);  
    params.prior = struct('alpha',ones(1,ndims));  
    params.likelihood_params = struct('distance_threshold',0.015,'jitter',0.1);
    params.lambda_gap = 10;
    truncate = 1e-30;

    fprintf(1,'\n\n\nStarting changepoint detection on %d data points with dimension %d\n',...
        size(data,1),ndims);

    [changepts, particle_history] = changepointdetect(data, params, @logdcmpdf, ...
            @update_stats_dcm, truncate);

    save(outf,'particle_history');
end

%plot particle history
h = plot_particle_history(particle_history, selected);